package com.shanda.sc.controller;

import java.io.Serializable;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Hex;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller(value = "callBackController")
@RequestMapping("/api/webhook")
public class CallBackController {

	

	class ResultData implements Serializable {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		/** 成功 */
		public static final int STATUS_SUCCESS = 1;
		/** 失败 */
		public static final int STATUS_ERROR = 0;

		/** 返回值的状态,可以为success或者error */
		private Integer code;

		/** 成功或者错误的信息 */
		private String message;

		private Map<String, Object> data = new HashMap<String, Object>();

		public ResultData() {
			super();
		}

		public ResultData(Integer code, String message) {
			super();
			this.code = code;
			this.message = message;
		}

		public ResultData(Integer code, String message, Map<String, Object> data) {
			super();
			this.code = code;
			this.message = message;
			this.data = data;
		}

		public Integer getCode() {
			return code;
		}

		public void setCode(Integer code) {
			this.code = code;
		}

		public String getMessage() {
			return message;
		}

		public void setMessage(String message) {
			this.message = message;
		}

		public Map<String, Object> getData() {
			return data;
		}

		public void setData(Map<String, Object> data) {
			this.data = data;
		}

		public void put(String key, Object value) {
			this.data.put(key, value);
		}

		public boolean isSuccess() {
			if (code != null && code == STATUS_SUCCESS) {
				return true;
			}
			return false;
		}
		
	}

		public boolean verify(String appkey, String token, long timestamp,
				String signature) throws NoSuchAlgorithmException,
				InvalidKeyException {
			Mac sha256HMAC = Mac.getInstance("HmacSHA256");
			SecretKeySpec secretKey = new SecretKeySpec(appkey.getBytes(),
					"HmacSHA256");
			sha256HMAC.init(secretKey);
			StringBuffer buf = new StringBuffer();
			buf.append(timestamp).append(token);
			String signatureCal = new String(Hex.encodeHex(sha256HMAC
					.doFinal(buf.toString().getBytes())));
			return signatureCal.equals(signature);
		}

		@RequestMapping(value = "/recieve.do")
		@ResponseBody
		public ResultData recieveMsg(Message message) {

			if (message == null) {
				return new ResultData(ResultData.STATUS_ERROR, "消息为空");
			}
			// 1.验证参数忽略

			// 2.验证签名
			try {
				String signature = message.getSignature();
				String appkey = "用户自己的key";
				String token = message.getToken();
				long timestamp = message.getTimestamp();
				if (!verify(appkey, token, timestamp, signature)) {
					return new ResultData(ResultData.STATUS_ERROR, "数据签名错误");
				}
			} catch (InvalidKeyException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchAlgorithmException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			// 3. 异步保存消息

			return new ResultData(ResultData.STATUS_SUCCESS, "成功");

		}
	

}

class Message implements Serializable{
	/**  */
	private static final long serialVersionUID = 1L;
	private String event;
	private String apiUser;
	private String category;
	private Integer labelId;
	private String emailId;
	private String maillistTaskId;
	private String mail_list_task_id;
	private String recipient;
	private long timestamp;
	private String token;
	private String signature;
	private String userHeaders;
	private Integer subStat;
	private String subStatDesc;
	public Message(){
		
	}
	
	public Message(String event, String apiUser, String category,
			Integer labelId, String emailId, String maillistTaskId,
			String mail_list_task_id, String recipient, long timestamp,
			String token, String signature, String userHeaders,
			Integer subStat, String subStatDesc) {
		super();
		this.event = event;
		this.apiUser = apiUser;
		this.category = category;
		this.labelId = labelId;
		this.emailId = emailId;
		this.maillistTaskId = maillistTaskId;
		this.mail_list_task_id = mail_list_task_id;
		this.recipient = recipient;
		this.timestamp = timestamp;
		this.token = token;
		this.signature = signature;
		this.userHeaders = userHeaders;
		this.subStat = subStat;
		this.subStatDesc = subStatDesc;
	}

	public String getEvent() {
		return event;
	}

	public void setEvent(String event) {
		this.event = event;
	}

	public String getApiUser() {
		return apiUser;
	}

	public void setApiUser(String apiUser) {
		this.apiUser = apiUser;
	}

	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}

	public Integer getLabelId() {
		return labelId;
	}

	public void setLabelId(Integer labelId) {
		this.labelId = labelId;
	}

	public String getEmailId() {
		return emailId;
	}

	public void setEmailId(String emailId) {
		this.emailId = emailId;
	}

	public String getRecipient() {
		return recipient;
	}

	public void setRecipient(String recipient) {
		this.recipient = recipient;
	}

	public long getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}

	public String getToken() {
		return token;
	}

	public void setToken(String token) {
		this.token = token;
	}

	public String getSignature() {
		return signature;
	}

	public void setSignature(String signature) {
		this.signature = signature;
	}

	public String getUserHeaders() {
		return userHeaders;
	}

	public void setUserHeaders(String userHeaders) {
		this.userHeaders = userHeaders;
	}

	public Integer getSubStat() {
		return subStat;
	}

	public void setSubStat(Integer subStat) {
		this.subStat = subStat;
	}

	public String getSubStatDesc() {
		return subStatDesc;
	}

	public void setSubStatDesc(String subStatDesc) {
		this.subStatDesc = subStatDesc;
	}

	public String getMaillistTaskId() {
		return maillistTaskId;
	}

	public void setMaillistTaskId(String maillistTaskId) {
		this.maillistTaskId = maillistTaskId;
	}

	public String getMail_list_task_id() {
		return mail_list_task_id;
	}

	public void setMail_list_task_id(String mail_list_task_id) {
		this.mail_list_task_id = mail_list_task_id;
	}

}
